clear all;
close all;

SimulationModel = 'IEEE_PSD_ESTIMATE_MODEL'; 
SimulationLength = 1000000; % Simulation Length in Bit Times

TypicalTransmitAmplitude = 1.2; % 1.2 V (2.4 Vpp) or 0.5 V (1.0 Vpp)

DataRate = 10e6; % Data Rate is 10 MBit/s
SymbolRate = DataRate * 3 / 4; % Symbol Rate is 7.5 MSymbols/s
Oversampling = 8; % 8 Times oversampled Data
Bins = 1025; % 1025 Frequency Bins
SimulationDuration = SimulationLength / DataRate;

% Simulate Typical Evaluation Board without Power Coupling Network
% Trapezoidal Signal with 25 % Rise/Fall Time
% Analog Transmitter Corner Frequency of 16 MHz
% Coupling Network (Corner Frequency approx. 16 MHz) equivalent to:
% 50 Ohm, 235 nF in each Line
% 100 pF between each Line and Shield (50 pF Line-to-Line)
% 600 W Surge Protection Diode with Bridge Rectifier
% No Power Coupling Network

Coefficients = [1 1 1 1] / 4; % FIR Pulse forming Coefficients
TransmitFilterFrequency = 16e6; % Transmit Filter Frequency
CouplingFilterFrequency = 16e6; % Coupling Network as described above
Amplitude = TypicalTransmitAmplitude; % Set Transmit Amplitude
Termination = 100; % 100 Ohms Termination Resistor
sim(SimulationModel); % Simulate Simulink Model
[PXX_TYP_WITHOUT, W] = pwelch(squeeze(TX), Bins, [], [], Oversampling * SymbolRate); % Calculate Welch Estimate of PSD
TYP_WITHOUT_PWR = 0; % Calculate Power Level
for I = 1 : 1 : Bins
    TYP_WITHOUT_PWR = TYP_WITHOUT_PWR + PXX_TYP_WITHOUT(I);
end
TYP_WITHOUT_PWR = mag2db(TYP_WITHOUT_PWR * (Oversampling * SymbolRate) / (2 * Bins)) / 2 + 30; % Convert Power Level into dBm
PXX_TYP_WITHOUT = mag2db(PXX_TYP_WITHOUT) / 2 + 30; % Convert Magnitude into dBm
plot(W / 1e6, PXX_TYP_WITHOUT, 'b') % Plot PSD Curve
xlabel('Frequency [MHz]')
ylabel('PSD [dBm/Hz]')
title('Power Spectral Density')
ylim([-120 -50])
hold on

% Simulate Typical Evaluation Board with Power Coupling Network
% Trapezoidal Signal with 25 % Rise/Fall Time
% Analog Transmitter Corner Frequency of 16 MHz
% Coupling Network (Corner Frequency approx. 8 MHz) equivalent to:
% 50 Ohm, 235 nF in each Line
% 100 pF between each Line and Shield (50 pF Line-to-Line)
% 600 W Surge Protection Diode with Bridge Rectifier
% Additional 250 uH coupled Inductor (RM5 core 3D3), in total 1 mH
% Additional 50 pF across each inductor (Simulation of Clamping Diodes)
% Additional 100 pF Line-to-Line (worst-case assumption)

Coefficients = [1 1 1 1] / 4;
TransmitFilterFrequency = 16e6; % Transmit Filter Frequency
CouplingFilterFrequency = 8e6; % Coupling Network as described above
Amplitude = TypicalTransmitAmplitude; % Set Transmit Amplitude
Termination = 100; % 100 Ohms Termination Resistor
sim(SimulationModel); % Simulate Simulink Model
[PXX_TYP_WITH, W] = pwelch(squeeze(TX), Bins, [], [], Oversampling * SymbolRate); % Calculate Welch Estimate of PSD
TYP_WITH_PWR = 0; % Calculate Power Level
for I = 1 : 1 : Bins
    TYP_WITH_PWR = TYP_WITH_PWR + PXX_TYP_WITH(I);
end
TYP_WITH_PWR = mag2db(TYP_WITH_PWR * (Oversampling * SymbolRate) / (2 * Bins)) / 2 + 30; % Convert Power Level into dBm
PXX_TYP_WITH = mag2db(PXX_TYP_WITH) / 2 + 30; % Convert Magnitude into dBm
plot(W / 1e6, PXX_TYP_WITH, 'b') % Plot PSD Curve

% Simulate Worst-Case Scenarios

% Trapezoidal Signal with 12.5 % Rise/Fall Time
% 20 MHz Transmit Filter (16 MHz + 25 %)
% 50 MHz Coupling Filter (minimum Capacitive Load)
% Increased Transmit Amplitude by 5 %
% 100 Ohms Termination Resistor

Coefficients = [1 1] / 2;
TransmitFilterFrequency = 20e6; % Transmit Filter Frequency
CouplingFilterFrequency = 50e6; % EMC Filter Frequency
Amplitude = TypicalTransmitAmplitude * 1.05; % Set Transmit Amplitude
Termination = 100; % 100 Ohms Termination Resistor
sim(SimulationModel); % Simulate Simulink Model
[PXX_WORST_CASE_TOP, W] = pwelch(squeeze(TX), Bins, [], [], Oversampling * SymbolRate); % Calculate Welch Estimate of PSD
WORST_CASE_TOP_PWR = 0; % Calculate Power Level
for I = 1 : 1 : Bins
    WORST_CASE_TOP_PWR = WORST_CASE_TOP_PWR + PXX_WORST_CASE_TOP(I);
end
WORST_CASE_TOP_PWR = mag2db(WORST_CASE_TOP_PWR * (Oversampling * SymbolRate) / (2 * Bins)) / 2 + 30; % Convert Power Level into dBm
PXX_WORST_CASE_TOP = mag2db(PXX_WORST_CASE_TOP) / 2 + 30; % Convert Magnitude into dBm
plot(W / 1e6, PXX_WORST_CASE_TOP, 'r') % Plot PSD Curve

% Trapezoidal Signal with 37.5 % Rise/Fall Time
% 12 MHz Transmit Filter (16 MHz - 25 %)
% 8 MHz Coupling Filter (see Power Coupling Network above)
% Reduced Transmit Amplitude by 5 %
% 100 Ohms Termination Resistor

Coefficients = [1 1 1 1 1 1] / 6;
TransmitFilterFrequency = 12e6; % Transmit Filter Frequency
CouplingFilterFrequency = 8e6; % EMC Filter Frequency
Amplitude = TypicalTransmitAmplitude * 0.95; % Set Transmit Amplitude
Termination = 100; % 100 Ohms Termination Resistor
sim(SimulationModel); % Simulate Simulink Model
[PXX_WORST_CASE_BOTTOM, W] = pwelch(squeeze(TX), Bins, [], [], Oversampling * SymbolRate); % Calculate Welch Estimate of PSD
WORST_CASE_BOTTOM_PWR = 0; % Calculate Power Level
for I = 1 : 1 : Bins
    WORST_CASE_BOTTOM_PWR = WORST_CASE_BOTTOM_PWR + PXX_WORST_CASE_BOTTOM(I);
end
WORST_CASE_BOTTOM_PWR = mag2db(WORST_CASE_BOTTOM_PWR * (Oversampling * SymbolRate) / (2 * Bins)) / 2 + 30; % Convert Power Level into dBm
PXX_WORST_CASE_BOTTOM = mag2db(PXX_WORST_CASE_BOTTOM) / 2 + 30; % Convert Magnitude into dBm
plot(W / 1e6, PXX_WORST_CASE_BOTTOM, 'r') % Plot PSD Curve